@@ -1,7 +1,12 @@
 var counter = 0,
   counter_unnamed = 0,
   editor,
+  editor2,
+  editor2isShown = false,
+  editorsAreStacked = true,
+  editorsHeight = 0.8 * window.innerHeight,
   modelInstances = [],
+  modelInstances2 = {},
   modelValues = {},
   modelStates = {},
   selectize,
